/* YAWN, Yet Another Web Navigator */
/* Copyright (C) 1998, James Troup <jjtroup@comp.brad.ac.uk>
 *                     Lee Willis <lwillis@comp.brad.ac.uk> */

/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>
#include "latin.h"
#include "utils.h"


/* latin.c
 * -------
 *
 * parse_latin_entities accepts a pointer to a null terminated string
 * of characters and removes latin character entities of the form
 * &name; or &#xxx as defined in the HTML 3.2 standard (Available at
 * http://www.w3.org/TR/REC-html32.html#latin1) and replacing them with
 * their proper values before passing the string back to the calling
 * function.
 *
 * NB. Currently only supports &quot; &lt; &gt; &amp; and their &#xxx
 *     equivalents.
 */
 

void parse_latin_entities(char *buffer)
{
  long index = 0;

  while (buffer[index] != '\0')
    
    {

      if (buffer[index] == '&')

	{
	
	  debug ( debug_parser, "latin.c: Found a & in the string %s", buffer+index);
	  
	  if (strncasecmp(buffer+index+1, "nbsp", 4) == 0 ||
	      strncmp(buffer+index+1, "#160", 4) == 0)
	    {
	      debug ( debug_parser, "latin.c: Found a &nbsp, replacing" ) ;
	      
	      /* Note currently this is a semi-broken implementation since it treats a &nbsp;
	       * exactly as a normal space character as the renderer (neve.c - Troup[98]) has 
	       * no concept of a non-breaking space. Does this matter? YARK
	       */

	      buffer[index] = ' '; /* Replace the & with a ' ' */
	      index++; /* Skip the newly added ' ' */
	      debug ( debug_parser, "latin.c: Now pointing at->%s", buffer+index);
	      if (buffer[index+4] == ';')
		strcpy (buffer+index, buffer+index+5); /* Shift the rest of the text over the nbsp; / #160; */
	      else
		strcpy (buffer+index, buffer+index+4); /* Shift the rest of the text over the nbsp / #160 */
	      debug ( debug_parser, "latin.c: Now pointing at->%s", buffer+index);
	    }

	  else if (strncasecmp(buffer+index+1, "amp", 3) == 0 ||
		   strncmp(buffer+index+1, "#38", 3) == 0 )
	    { 
	      debug ( debug_parser, "latin.c: Found an &amp ... replacing.");
	      index++; /* Skip the & */
	      debug ( debug_parser, "latin.c: Now pointing at->%s", buffer+index);
	      if (buffer[index+3] == ';')
		strcpy(buffer+index, buffer+index+4); /* Shift the rest of the text over the amp; / #38; */
	      else
		strcpy(buffer+index, buffer+index+3); /* Shift the rest of the text over the amp / #38 */
	      debug ( debug_parser, "latin.c: Now pointing at->%s", buffer+index);
	    }
	  
	  else if (strncasecmp(buffer+index+1, "gt", 2) == 0 ||
		   strncmp(buffer+index+1, "#62", 3) == 0 )
	    {
	      debug ( debug_parser, "latin.c: Found an &gt; ... replacing.");
	      buffer[index] = '>'; /* Replace the & with a > */
	      index++; /* Skip the newly created > */
	      debug ( debug_parser, "latin.c: Now pointing at->%s", buffer+index);
	      if (buffer[index]=='g')
		{
		  if (buffer[index+2] == ';')
		    strcpy(buffer+index, buffer+index+3); /* Shift the rest of the text over the gt; */
		  else
		    strcpy(buffer+index, buffer+index+2); /* Shift the rest of the text over the gt */
		  debug ( debug_parser, "latin.c: Now pointing at->%s", buffer+index);
		}
	      else
		{
		  if (buffer[index+3] == ';')
		    strcpy(buffer+index, buffer+index+4); /* Shift the rest of the text over the #62; */
		  else
		    strcpy(buffer+index, buffer+index+3); /* Shift the rest of the text over the #62 */
		  debug ( debug_parser, "latin.c: Now pointing at->%s", buffer+index);
		}
	    }
	  
	  else if (strncasecmp(buffer+index+1, "lt", 2) == 0 ||
		   strncmp(buffer+index, "#60", 3) == 0 )
	    {
	      debug (debug_parser, "latin.c: Found an &lt; ... replacing.");
	      buffer[index] = '<'; /* Replacing the & with a < */
	      index++; /* Skip the newly created < */
	      debug (debug_parser, "latin.c: Now pointing at->%s", buffer+index);
	      if (buffer[index]=='l')
		{
		  if (buffer[index+2] == ';')
		    strcpy(buffer+index, buffer+index+3); /* Shift the rest of the text over the lt; */
		  else
		    strcpy(buffer+index, buffer+index+2); /* Shift the rest of the text over the lt */
		  debug (debug_parser, "latin.c: Now pointing at->%s", buffer+index);
		}
	      else
		{
		  if (buffer[index+3] == ';')
		    strcpy(buffer+index, buffer+index+4); /* Shift the rest of the text over the #60; */
		  else
		    strcpy(buffer+index, buffer+index+3); /* Shift the rest of the text over the #60 */
		  debug ( debug_parser, "latin.c: Now pointing at->%s", buffer+index);
		}
	    }

	  else if (strncasecmp(buffer+index+1, "copy", 4) == 0 ||
		   strncmp(buffer+index, "#169", 4) == 0 )
	    {
	      debug (debug_parser, "latin.c: Found an &copy ... replacing." );
	      buffer[index] = 169; /* Insert the copyright symbol */
	      index++; /* Skip thenewly created (c) */
	      debug (debug_parser, "latin.c: Now pointing at ->%s", buffer+index);
	      if (buffer[index] == 'c')
		{
		  if (buffer[index+4] == ';')
		    strcpy(buffer+index, buffer+index+5); /* Shift the rest of the text over the copy; */
		  else
		    strcpy(buffer+index, buffer+index+4); /* Shift the rest of the text over the copy; */
		  debug (debug_parser, "latin.c: Now pointing at->%s", buffer+index);
		}
	      else
		{
		  if (buffer[index] == ';')
		    strcpy(buffer+index, buffer+index+5); /* Shift the rest of the text over the #60; */
		  else
		    strcpy(buffer+index, buffer+index+4); /* Shift the rest of the text over the #60; */
		  debug ( debug_parser, "latin.c: Now pointing at->%s", buffer+index);
		}
	    }
	  
	  else if (strncasecmp(buffer+index+1, "#32", 3) == 0)
	    {
	      debug (debug_parser, "latin.c: Found a &#32 ... replacing." );
	      buffer[index] = 32;
	      index++;
	      debug (debug_parser, "latin.c: Now pointing at ->%s", buffer+index);
	      if (buffer[index+3] == ';')
		strcpy(buffer+index, buffer+index+4);
	      else
		strcpy(buffer+index, buffer+index+3);
	      debug ( debug_parser, "latin.c: Now pointing at->%s", buffer+index);
	    }
	  
	  else
	    index++;
	  /* Unrecognized entities just rendered as they appear in the source */
	  
	}
      
      else
	index++; /* Skip onto the next bit of the text. */
	
    }

}
