/* YAWN, Yet Another Web Navigator */
/* Copyright (C) 1998, James Troup <jjtroup@comp.brad.ac.uk>
 *                     Lee Willis <lwillis@comp.brad.ac.uk> */

/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include "utils.h"
#include "neve.h"
#include "neve_operations.h"


/* neve_operations.c
 * -----------------
 *
 * Contains various functions for manipulation and output of the neve
 * structure.
 * out_neve_type - Outputs to standard error via the debug procedure the
 *               - type name of the neve_node pointed to by its argument.
 * output_list   - Outputs the neve structure pointed to by its argument.
 * new_neve_node - Creates a new neve_node at the end of the current structure
 *               - with the type as given by the functions argument.
 */


void out_neve_type (neve_in_chains *node)
{
  switch (node->type)
    {
    case text:
      debug ( debug_parser, "TEXT:");
      break;
    case image:
      debug ( debug_parser, "IMAGE:");
      break;
    case horizontal_rule:
      debug ( debug_parser, "HR:");
      break;
    case line_break:
      debug ( debug_parser, "LB:");
      break;
    case paragraph_break:
      debug ( debug_parser, "PB:");
      break;
    case title:
      debug ( debug_parser, "TITLE:");
      break;
    case end_link:
      debug ( debug_parser, "</A>:");
      break;
    case start_list_element:
      debug ( debug_parser, "<LI>:");
      break;
    case undefined:
      debug ( debug_parser, "Undefined type:");
      break;
    default:
      debug (debug_parser, "We shouldn't ever get here, if we have the neve node\n\
set up without a valid type field.");
      break;
    }
}



void output_list(neve_in_chains *temp)
{

  while (temp) { /*While there is still some structure to be output */

    out_neve_type (temp); /* Ouput the node type */
    
    debug (debug_parser, "    link? - %s",
	   temp->logical_attributes.link?"True":"False");
    
    if (temp->type == text || temp->type == title)
      debug (debug_parser, "    $$%s$$",
	     temp->buf?temp->buf:"Null buffer!");
    
    if (temp->logical_attributes.link || temp->type == image)
      debug (debug_parser, "    href  = %s",
	     temp->href ? temp->href : "Broken!");

    if (temp->type == image)
      debug (debug_parser, "    src = %s",
	     temp->src ? temp->src : "Broken!");

    if (temp->logical_attributes.ul_list_element)
      debug (debug_parser, "    UL Element, Indent = %d, Number = %d", 
	     temp->logical_attributes.list_indent_level, temp->logical_attributes.item_number);

    if (temp->logical_attributes.ol_list_element)
      debug (debug_parser, "    OL Element, Indent = %d, Number = %d", 
	     temp->logical_attributes.list_indent_level, temp->logical_attributes.item_number);

    if (temp->logical_attributes.dl_list_element)
      debug (debug_parser, "    DL Element, Indent = %d, Number = %d", 
	     temp->logical_attributes.list_indent_level, temp->logical_attributes.item_number);

    debug (debug_parser, "    align = %d", temp->physical_attributes.alignment);
    debug (debug_parser, "    size = %d", temp->physical_attributes.font_size);
    debug (debug_parser, "    colour = %d.%d.%d", 
	   temp->physical_attributes.font_colour.red,
	   temp->physical_attributes.font_colour.green,
	   temp->physical_attributes.font_colour.blue);

    temp = temp->next;
  }
}



void new_neve_node(neve_types new_node_type)
{

  if (!neve_list) {
    /* Generate a structure if it hasn't already be created */
    neve_list = xmalloc(sizeof(*neve_list));
    neve_head = neve_list;
    neve_list->buf = NULL;
    neve_list->href = NULL;
    neve_list->src = NULL;
    neve_list->type = new_node_type;
    neve_list->physical_attributes = current_pattributes;
    neve_list->logical_attributes = current_lattributes;
    neve_list->next = NULL;
    neve_list->previous = NULL;
  }
  
  if (neve_list->type == undefined)
    /* If the current node is undefined ie has no displayable content so far
     * it will however contain status information such as font size changes etc.
     * instead of creating a new node therefore we can simply change the type of
     * this one.
     */
    {
      neve_list->type = new_node_type;
      debug ( debug_parser, "Converted Undef to ..." );
      out_neve_type (neve_list);
    }
  else 
    /* We need to xmalloc the space for a new node and set it up with the
     * relevant state information */
    {
      neve_list->next = xmalloc (sizeof(*neve_list));
      neve_list->next->previous = neve_list;
      neve_list = neve_list->next;
      neve_list->buf = NULL;
      neve_list->href = NULL;
      neve_list->src = NULL;
      neve_list->type = new_node_type;
      neve_list->physical_attributes = current_pattributes;
      neve_list->logical_attributes = current_lattributes;
      neve_list->next = NULL;
      
      debug (debug_parser, "Added ..." );
      out_neve_type (neve_list); /* Only output if we're debugging */
  }
  
}


