/* YAWN, Yet Another Web Navigator */
/* Copyright (C) 1998, James Troup <jjtroup@comp.brad.ac.uk>
 *                     Lee Willis <lwillis@comp.brad.ac.uk> */

/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include "parse_attribs.h"
#include "utils.h"
#include "tagdefs.h"
#include "neve.h"
#include "staxx.h"


/* parse_attribs.c deals with the parsing of the attributes within HTML tags
 * it also decides whether the information provided makes sense within the
 * current tag context and if so where it should put the resulting information
 * 
 * Each function documented seperately
 */


char foo[10]; /* Globals suck, but if we don't stick it here we end up
	       * declaring it every function seems fundamentally naff
	       * at this moment in time
	       */

int get_bit (char *document, long *index)
{
  int temp = 0;
  foo[9] = '\0';
  
  if (strlen(document+(*index)))
    {
      strncpy (foo, document+(*index), 
	       temp = strlen(document+(*index)) < 9 ? temp : 9);
      return 1;
    }
  else
    return 0;
}



int skip_equals(char *document, long *index) 
{
  /* Skips optional whitespace followed by an '=' followed by more
   * optional whitespace.
   */

  debug (debug_parse_attribs, "skip_equals: entering pointing at %ld:%s...", 
	 *index, get_bit(document, index) ? foo  : "NULL");
  
  while (is_whitespace(document[*index]))
    (*index)++;
  
  if (document[*index]!='=')
    return -1;
  
  (*index)++;
  
  while (is_whitespace(document[*index]))
    (*index)++;
  
  debug (debug_parse_attribs, "skip_equals: leaving pointing at %ld:%s...",      
	 *index, get_bit (document, index) ? foo : "NULL");

  
  return 0;
}



void handle_src (char *document, long *index, neve_in_chains *node, int tag)                          
{
  /* Parses src="foo bar" and src="foobar" constructs and decides what
   * to do with them. ie src makes no sense inside an <BASEFONT> tag so we
   * ignore it. Only makes sense inside an anchor tag.
   */
  int quoted = 0;
  int src_counter = 0;
  
  debug (debug_parse_attribs, "handle_src: entering pointing at %ld:%s...", 
	 *index, get_bit (document, index) ? foo : "NULL");
  
  (*index)+=3; /* Skip SRC */
  
  if (skip_equals(document, index) == -1)
    return;

  if (document[*index]=='"') 
    { /* Recognize if we're href="foo vr" type and skip
       * opening " */
      debug (debug_parse_attribs, "handle_src: found \".");
      quoted = 1;
      (*index)++;
    }
  
  /* Count how long the src information is. Terminated by a " if we're
   * parsing a quoted form or a ' ' if we're not
   */
  while (((document[*index] != ' ' && document[*index] != '>') && !quoted) || 
	 (document[*index] != '"' && quoted))
    {
      src_counter++;
      (*index)++;
    }

  debug (debug_parse_attribs, "handle_src: src information is %d characters.", src_counter);
  
  /* Create space for the src information and copy it into the current node */
  if (tag == ANCHOR || tag == IMAGE)
    {
      debug (debug_parse_attribs, "handle_src: Copying into node.");
      node->src = xmalloc (src_counter+1);
      strncpy(node->src, document+*index-src_counter, src_counter);
      node->src[src_counter] = '\0';
    }
  
  /* Skip the close quote if we're present.
   * NB this doesn't check the quoted flag so that we can successfully 
   * deal with <IMG src="foobar.gif> without falling over.
   */
  if (document[*index]=='"') 
    (*index)++;

  /* Skip whitespace until end of tag */
  while ((is_whitespace(document[*index])) && document[*index] != '>')
    (*index)++;

  debug (debug_parse_attribs, "handle_src: Leaving pointing at %ld:%s...", 
	 *index, get_bit (document, index) ? foo : "NULL");
}



void handle_href (char *document, long *index, neve_in_chains *node, int tag)
{
  /* Parses href="foo bar" and href="foobar" constructs and decides what
   * to do with them.  Essentially the same as handle_src (Above.)
   */
  int quoted = 0;
  int href_counter = 0;
  
  debug (debug_parse_attribs, "handle_href: Entering pointing at %ld:%s...",
	 *index, get_bit (document, index) ? foo : "NULL");
  (*index)+=4; /* Skip HREF */
  debug (debug_parse_attribs, "handle_href: Skipped HREF, now pointing at %ld:%s...",
	 *index, get_bit (document, index) ? foo : "NULL");
  
  if (skip_equals(document, index) == -1)
    return;
  
  if (document[*index]=='"') 
    {
      debug (debug_parse_attribs, "handle_href: Value is quoted.");
      quoted = 1;
      (*index)++;
    }
  
  /* Skip until we find a space or end of tag > if we're not quoted or until we
   * find a " if we are quoted */
  while (((document[*index] != ' ' && document[*index] != '>') && !quoted) || 
	 (document[*index] != '"' && quoted))
    {
      href_counter++;
      (*index)++;
    }
  
  debug (debug_parse_attribs, "handle_href: HREF info is %d characters.", 
	 href_counter);

  if (tag == ANCHOR || tag == IMAGE) 
    {
      node->href = xmalloc (href_counter+1);
      strncpy(node->href, document+*index-href_counter, href_counter);
      node->href[href_counter] = '\0';
      debug (debug_parse_attribs, "handle_href: HREF info is %s", 
	     node->href ? node->href : "NULL");
    }
  
  if (document[*index]=='"')
    (*index)++;

  /* Skip whitespace until end of tag */
  while ((is_whitespace(document[*index])) && document[*index] != '>')
    (*index)++;
  
  debug (debug_parse_attribs, "handle_href: Leaving pointing at %ld:%s...", 
	 *index, get_bit(document, index) ? foo : "NULL");

}




void handle_align (char *document, long *index, neve_in_chains *node, int tag,
		   physical_attribute_types *current_pattributes)
{
  /* Parse align=center/left/right/top/middle/bottom for headers, paragraph,
   * horizontal rule and image tags.
   * NB. Some of these don't make sense according to the standard so we have
   * to be careful about if we copy the alignment over
   */
  debug (debug_parse_attribs, "handle_align: Entering pointing at %ld:%s...", 
	 *index, get_bit (document, index) ? foo : "NULL");
  
  (*index)+=5; /* Skip ALIGN */

  debug (debug_parse_attribs, "handle_align: Skipped ALIGN, now pointing at %ld:%s...", 
	 *index, get_bit (document, index) ? foo : "NULL");
  
  if (skip_equals(document, index) == -1)
    return;
  
  /* Yark should we check for quotes? I think so, it's not in the standard but
   * hey people consistently write daft HTML so we'll check it just in case
   */
  
  if (document[*index] == '"')
    {
      debug (debug_parse_attribs, "handle_align: Found open quote, skipping.");
      (*index)++;
    }
  
  if ((tag >= H1 && tag <=H6) || tag==PARAGRAPH || tag==DIV || 
      tag==HRULE || tag == IMAGE)
    {
      
      if (strncasecmp(&(document[*index]), "left", 4) == 0)
	{
	  /* Allowable for all the above cases */
	  debug (debug_parse_attribs, "handle_align: left found.");
	  node->physical_attributes.alignment = left;
	  (*index) += 4;
	}
      else if (strncasecmp(&(document[*index]), "right", 5) == 0)
	{
	  /* Allowable for all the above cases */
	  debug (debug_parse_attribs, "handle_align: right found.");
	  node->physical_attributes.alignment = right;
	  (*index) += 5;
	}
      else if (strncasecmp(&(document[*index]), "center", 6) == 0)
	{
	  debug (debug_parse_attribs, "handle_align: center found.");
	  if (tag != IMAGE) /* IMG isn't supposed to have an align=center */
	    node->physical_attributes.alignment = centre;
	  else
	    debug ( debug_parse_attribs, "Ignored.");
	  (*index) += 6;
	}
      else if (strncasecmp(&(document[*index]), "bottom", 6) == 0)
	{
	  debug (debug_parse_attribs, "handle_align: bottom found.");
	  if (tag == IMAGE) /* bottom, middle and top are only applicable to
			   * IMG tags. */
	    node->physical_attributes.alignment = bottom;
	  else
	    debug ( debug_parse_attribs, "Ignored.");
	  (*index) += 6;
	}
      else if (strncasecmp(&(document[*index]), "middle", 6) == 0)
	{
	  debug (debug_parse_attribs, "handle_align: middle found.");
	  if (tag == IMAGE)
	    node->physical_attributes.alignment = middle;
	  else
	    debug (debug_parse_attribs, "Ignored.");
	  (*index) += 6;
	}
      else if (strncasecmp(&(document[*index]), "top", 3) == 0)
      {
	debug (debug_parse_attribs, "handle_align: top found.");
	if (tag == IMAGE)
	  node->physical_attributes.alignment = top;
	else
	  debug (debug_parse_attribs, "Ignored.");
	(*index) += 3;
      }
      
      if (tag != HRULE && tag != IMAGE) 
	{
	  /* Only carry the alignments given here through to subsequent elements if
	   * we're in a paragraph, header or a div. It makes no sense to
	   * carry it thorugh if this is an image or horizontal rule */
	  current_pattributes->alignment = node->physical_attributes.alignment;
	}
    }

  if (document[*index] == '"')
    (*index)++;
  
  /* Skip whitespace until end of tag */
  while ((is_whitespace(document[*index])) && document[*index] != '>')
    (*index)++;
  
  debug (debug_parse_attribs, "handle_align: Leaving pointing at %ld:%s...", 
	 *index, get_bit (document, index) ? foo : "NULL");
}




void handle_unrecognized (char *document, long *index)
{
  /* Skip an attribute if we don't recognize it. We'll assume 
   * that it goes name[ ]{=[ ]value} */
  
  while ((!(is_whitespace(document[*index])) && 
	  (document[*index] != '='))) /* Skip name */
    (*index)++;
  
  while (is_whitespace(document[*index]) ) /* Skip whitespace */
    (*index)++;
  
  if ( document[*index]=='=' ) 
    { /* If it has an assignment part then skip that ...*/
      
      (*index)++; /* Skip the = */
      
      while (is_whitespace(document[*index])) 
	/* Skip optional whitespace after the = but before the value */
	(*index)++;
      
      while ((!(is_whitespace(document[*index]))) && (document[*index] != '"'))
	/* Skip the value, assume no spaces */
	(*index)++;
      
      if (document[*index]=='"' )
	{ /* Skip any quoted text */
	  (*index)++;
	  while (document[*index] != '"' )
	    (*index)++;
	  (*index)++; /* Assume that the second " is the closing part 
		       * of the value and so carry on now 
		       */
	}
    }	
}



long handle_value (char *document, long *index)
{
  /* Convert a string of numbers to their decimal value, just keeps
   * going until we hit a non-digit. YARK This is probably
   * very evil, but it should work in 99% of vaguely sane cases
   */
  long running_total = 0;
  
  debug (debug_parse_attribs, "handle_value: Entering pointing at %ld:%s...", 
	 *index, get_bit (document, index) ? foo : "NULL");
  
  while (isdigit(document[*index])) 
    {
      running_total *= 10;
      running_total += document[*index] - '0'; /* Safe for ASCII, should be others */
      debug (debug_parse_attribs, "handle_value: Running Total = %ld.", running_total);
      (*index)++;
    }
  
  debug ( debug_parse_attribs, "handle_value: Returning %ld, now pointing at %ld:%s...",
	  running_total, *index, get_bit (document, index) ? foo : "NULL");
  
  return running_total;
}



void handle_size (char *document, long *index,neve_in_chains *node, int tag, 
		  int *basefont, int *font_change_implemented, 
		  physical_attribute_types *current_pattributes)
{     
  /* Handle the size=dd, size="dd", size=+1, size ="+1" cases
   * This is not nice to handle, but I'm pretty sure this works now
   */
  int size = *basefont;  /* Default the size to basefont so when we get
			  * a relative size it adds on correctly */
  (*index)+=4; /* Skip SIZE */
  if (skip_equals(document, index) == -1)
    return;
  debug (debug_parse_attribs,"handle_size: Checking for quoted values.");
  if (document[*index] == '"')
    {
      (*index)++;
      debug (debug_parse_attribs, "handle_size: \" found.");
    }
  if ((document[*index] == '+' || document[*index] == '-'))
    {
      debug ( debug_parse_attribs,"Found a relative value %c", document[*index]);
      if (tag == FONT || tag == HRULE)
	{
	  if (document[*index] == '-')
	    {
	      (*index)++; /* Skip - */
	      size = *basefont - handle_value (document, index);
	      debug (debug_parse_attribs, "handle_size: size now equals %d.", size);
	    }
	  else 
	    {
	      (*index)++; /* Skip + */
	      size = *basefont + handle_value (document, index);
	      debug (debug_parse_attribs, "handle_size: size now equals %d.", size);
	    }
	}
      else
	{
	  handle_value (document, index); /* Get the value but ignore it */
	  debug (debug_parse_attribs, 
		 "handle_size: ignoring value, relative value makes no sense in tag type %d",
		 tag);
	}
      if (document[*index] == '"') /* Skip closing quote if it's there */
	(*index)++;
    }
  else /* Surely we don't need this -> if (document[*index] != '+' && document[*index] != '-') */
    {
      size = handle_value (document, index);
      debug (debug_parse_attribs, "handle_size: size now equals %d.", size);
    }
  
  if (tag == FONT || tag == BASEFONT) 
    
    {
      
      if (size  > 7) /* Make sure the size is a valid range for the FONT
		      * and BASEFONT tags.
		      */
	size = 7;
      
      else if (size < 0)
	
	size = 0;
      
      if (tag == FONT)
	if (!(*font_change_implemented))
	  {
	    debug(debug_parse_attribs, "handle_size: creating new font stack node.");
	    push_font (node->previous, FALSE, TRUE );
	    *font_change_implemented = TRUE;
	  }
	else
	  {
	    /* We've already made one change so we want to adjust the top node
	     * of the stack not create a new one. See staxx.c for more details
	     */
	    debug (debug_parse_attribs, "handle_colour: Hacking top font stack node.");
	    hack_fstack_size ();
	  }
      
      node->physical_attributes.font_size = size; /* Alter the font size */
      current_pattributes->font_size = size;
      
      if (tag == BASEFONT)
	*basefont = size;
    }
  else if (tag == HRULE) {
    
    node->physical_attributes.height = size;  /* Place the correct size into our node
					       * NB, don't bounds check or handle % values
					       * correctly, well at all to be perfectly
					       * honest
					       */
    }
  
}



int hex_to_dec (char s)
{
  /* Returns the decimal value of a hex digit */
  if (isdigit(s))
    return s-'0';
  else if (s >= 'a' && s <= 'f')
    return 10+s-'a';
  else if (s >= 'A' && s <= 'F')
    return 10+s-'A';
  return -1; /* Keep gcc happy */
}



int convert_hex (char *doc, long *index)
{
  /* Return the hex value of a two character sequence provided they
   * are both valid hex digits
   */
  int value;
  
  if (isxdigit(doc[*index]) && isxdigit(doc[*index+1])) {
    
    value = 16 * (hex_to_dec(doc[*index]));
    (*index)++;
    
    value += hex_to_dec(doc[*index]);
    (*index)++;
    
    return value;
  
  }
  
  else
  
    return -1;
}
	
	

void handle_colour(char *document, long *index, neve_in_chains *node, 
		   int tag, int *font_change_implemented, 
		   physical_attribute_types *current_pattributes)
{
  /* parse the color="#xxxxxx" structure, should also parse the common
   * color names, but erm doesn't yet.  
   */
  colour_type value = {-1,-1,-1};
  
  (*index)+=5; /* Skip COLOR */
  
  if (skip_equals (document, index) == -1)
    return;
  
  if (document[*index] == '"')
    (*index)++;
  
  if (document[*index] == '#') 
    {
      (*index)++; /* Skip the # */
      
      if ((value.red = convert_hex (document, index)) == -1)
	return;
      debug ( debug_parse_attribs, "handle_colour: Red value = %d", value.red);  
      
      if ((value.green = convert_hex (document, index)) == -1)
	return;
      debug ( debug_parse_attribs, "handle_colour: Green value = %d", value.green);
      
      if ((value.blue = convert_hex (document, index)) == -1)
	return;
      debug ( debug_parse_attribs, "handle_colour: Blue value = %d", value.blue);
      
      if (document[*index] =='"')
	(*index)++;
      
      if (tag == FONT) /* Doesn't make any sense for any other tag */
	{
	  if (!(*font_change_implemented))
	    {
	      debug ( debug_parse_attribs, 
		      "handle_colour: Creating new font stack node.");
	      push_font (node, TRUE, FALSE); /* Create a new font stack node */
	    }
	  else
	    {
	      debug (debug_parse_attribs, 
		     "handle_colour: Hacking top font stack node.");
	      hack_fstack_colour(); /* We've already had a font attirbute change in this
				     * tag so just change the top node's value
				     */
	    }
	  node->physical_attributes.font_colour = value;
	  current_pattributes->font_colour = value;
	}
    }
  else
    {
      ; /* Should check the named colors here */
    }
}



void handle_alt (char *document, long *index, neve_in_chains *node, int tag)
{
  /* handles the ALT=alttext and ALT="alt text" attributes for the IMG tag,
   * NB Makes no sense to anything else.
   */
  
  int quoted = 0;
  long buf_counter = 0;
  
  debug (debug_parse_attribs, "handle_alt: Entering pointing at %ld:%s...",
	 *index, get_bit (document, index)  ? foo : "NULL" );

  (*index) += 3; /* Skip ALT */

  debug (debug_parse_attribs, "handle_alt: Skipped ALT, now pointing at %ld:%s",
	 *index, get_bit (document, index)  ? foo : "NULL" );
  
  if (skip_equals (document, index) == -1)
    return;
  
  if (document[*index]=='"') {
    quoted = 1;
    (*index)++;
  }
  
  /* Skip until we hit whitespace or end of tag if we're unquoted or
   * until we find a close quote if we're not quoted.
   */
  while (((document[*index] != ' ' && document[*index] != '>') && !quoted) || 
	 ((document[*index] != '"') && quoted))
    {
      buf_counter++;
      (*index)++;
    }
  
  debug (debug_parse_attribs, "handle_alt: ALT info is %ld characters long.",
	 buf_counter);
  
  if (tag == IMAGE) /* Makes no sense for anything else */
    {
      node->buf = xmalloc(buf_counter+1);
      strncpy(node->buf, document+*index-buf_counter, buf_counter);
      node->buf[buf_counter] ='\0';
      debug (debug_parse_attribs, "handle_alt: Info is %s",
	     node->buf);
    }
  
  if (document[*index]=='"')
    (*index)++;

  debug (debug_parse_attribs, "handle_alt: Leaving pointing at %ld:%s...",
	 *index, get_bit (document, index)  ? foo : "NULL" );

}


void handle_type (char *document, long *index, neve_in_chains *node, int tag)
{
  /* Handle the type = disc/square/circle/1/a/A/i/I for determining the
   * list bullet types or numbering type
   */

  debug ( debug_parse_attribs, "handle_type: Entering pointing at %ld:%s...",
	  *index, get_bit(document, index) ? foo : "NULL");

  (*index) += 4;

  debug ( debug_parse_attribs, "handle_type: Skipped TYPE, now pointing at %ld:%s...",
	  *index, get_bit(document, index) ? foo : "NULL");

 if (skip_equals (document, index) == -1 )
   return;

 if (tag == OLIST) /* The following make no sense in any other tag */
   {
     if (document[*index] == '1')
       {
	 debug (debug_parse_attribs, "handle_type: Set type to 1,2,3 etc.");
	 node->logical_attributes.ol_number_type = arabic;
	 (*index)++;
       }
     else if (document[*index] == 'a')
       {
	 debug (debug_parse_attribs, "handle_type: Set type to a,b,c etc.");
	 node->logical_attributes.ol_number_type = lalpha;
	 (*index)++;
       }
     else if (document[*index] == 'A')
       {
	 debug (debug_parse_attribs, "handle_type: Set type to A,B,C etc.");
	 node->logical_attributes.ol_number_type = ualpha;
	 (*index)++;
       }
     else if (document[*index] == 'i')
       {
	 debug (debug_parse_attribs, "handle_type: Set type to i,ii,iii etc.");
	 node->logical_attributes.ol_number_type = lroman;
	 (*index)++;
       }
     else if (document[*index] == 'I')
       {
	 debug (debug_parse_attribs, "handle_type: Set type to I, II, III etc.");
	 node->logical_attributes.ol_number_type = uroman;
	 (*index)++;
       }
     else 
       {
	 debug (debug_parse_attribs, "handle_type: defaulting to 1,2,3 etc.");
	 node->logical_attributes.ol_number_type = 1; /* Default to 1,2,3 */
       }
   }
 else if (tag == ULIST) /* The following make no sense in any other tag */
   {
     if (strncasecmp(document+*index, "disc", 4) == 0)
       {
	 debug (debug_parse_attribs, "handle_type: bullet type is now disc.");
	 node->logical_attributes.ul_bullet_type = disc;
	 (*index)++;
       }
     else if (strncasecmp(document+*index, "square", 6) ==0)
       {
	 debug (debug_parse_attribs, "handle_type: bullet type is now square.");
	 node->logical_attributes.ul_bullet_type = square;
	 (*index)++;
       }
     else if (strncasecmp(document+*index, "circle", 6) ==0)
       {
	 debug (debug_parse_attribs, "handle_type: bullet type is now circle.");
	 node->logical_attributes.ul_bullet_type = circle;
	 (*index)++;
       }
     else
       {
	 debug (debug_parse_attribs, "handle_type: defaulting to disc.");
	 node->logical_attributes.ul_bullet_type = 1; /* Default to disc */
       }
   }
 while (document[*index] && is_whitespace(document[*index]))
   (*index)++;
}


void handle_width (char *document, long *index, neve_in_chains *node, int tag)
{
  /* Handle the width attribute for various tags */
  
  int quoted = 0;
  long width;
  
  debug (debug_parse_attribs, "handle_width: Entering pointing at %ld:%s...",
	 *index, get_bit(document,index) ? foo : "NULL");
  
  (*index) += 5; /*Skip WIDTH */

  debug (debug_parse_attribs, "handle_width: Skipped WIDTH, now pointing at %ld:%s...",
	 *index, get_bit(document,index) ? foo : "NULL");
  
  if (skip_equals(document, index) == -1 )
    return;
  
  if ((document[*index]=='"'))
    {
      debug (debug_parse_attribs, "handle_width: Found \".");
      (*index)++;
      quoted = 1;
    }
  
  if (isdigit(document[*index]))
    width = atoi (&(document[*index])); /* Standard library function, need to check if thi
					 * is robust enough for us */
  
  /* Skip until we find a space or end of tag if we're not
   * quoted or until we find a close quote if we're quoted
   */
  while (((document[*index] != ' ' && document[*index] != '>') && !quoted) ||
	 ((document[*index] != '"') && quoted))
    (*index)++;
  
  if (document[*index] == '"')
    (*index)++;
  
  if (tag == PRE || tag == HRULE || tag == IMAGE || tag == APPLET)
    {
      debug ( debug_parse_attribs, "handle_width: Setting width to %ld.", width);
      node->physical_attributes.width = width;
    }
  
  debug (debug_parse_attribs, "handle_width: Leaving pointing at %ld:%s...",
	 *index, get_bit(document,index) ? foo : "NULL");
  
}



void handle_height (char *document, long *index, neve_in_chains *node, int tag)
{
  /* Essentially the same as handle_width, see above for commentary */
  int quoted = 0;
  long height;
  
  debug (debug_parse_attribs, "handle_height: Entering pointing at %ld:%s...",
	 *index, get_bit(document,index) ? foo : "NULL");
  
  (*index) += 6; /*Skip HEIGHT */

  debug (debug_parse_attribs, "handle_width: Skipped HEIGHT, now pointing at %ld:%s...",
	 *index, get_bit(document,index) ? foo : "NULL");
  
  if (skip_equals(document, index) == -1 )
    return;
  
  if (document[*index]=='"')
    {
      debug ( debug_parse_attribs, "handle_height: Found a \".");
      quoted = 1;
      (*index)++;
    }
  
  if (isdigit(document[*index]))
    height = atoi (&(document[*index]));
  
  while (((document[*index] != ' ' && document[*index] != '>') && !quoted) ||
	 (document[*index] != '"' && quoted))
    (*index)++;
  
  if (document[*index] == '"')
    (*index)++;

  if (tag == IMAGE || tag == APPLET)
    {
      node->physical_attributes.height = height;
      debug (debug_parse_attribs, "handle_height: Setting height to %ld", height);
    }
  
  debug (debug_parse_attribs, "handle_width: Leaving pointing at %ld:%s...",
	 *index, get_bit(document,index) ? foo : "NULL");
  
}




void parse_attribs (char *document,long *index, neve_in_chains *neve_node, 
		    int tag, int *current_basefont, physical_attribute_types *current_pattribs)
{
  /* This is the main function which loops along the tags parametes, mathching
   * them if possible and calling the relevant functions.
   * Should be pointing at the space after the tag name or the closing > on entry 
   */
  
  int font_change_implemented = 0; /* Flag to record if we've changed any FONT attributes
				    * in this invocation so we know whether to create a new
				    * font_stack node or just add the information to the
				    * top node */
  if (document[*index] == '\0') /* Check for end of doc */
    return;
  
  if (document[*index] == '>')
    {
      debug (debug_parse_attribs, "parse_attribs: Found end of tag.");
      (*index)++;
      return;
    }
  
  debug (debug_parse_attribs, "parse_attribs: Entering main loop pointing at %ld:%s...",
	 *index, get_bit(document, index) ? foo : "NULL");
  
  while (document[*index] != '\0' && document[*index] != '>') {
        
    /* Skip whitespace before the attribute name */
    while (document[*index] != '\0' && document[*index] == ' ')
      (*index)++;
    
    /* Try to match the attribute name */
    if (strncasecmp(&document[*index], "alt", 3) == 0)
      {
	debug (debug_parse_attribs, "parse_attribs: calling handle_alt ...");
	handle_alt(document, index, neve_node, tag);
      }
    
    else if (strncasecmp(&document[*index], "href", 4) == 0)
      {
	debug (debug_parse_attribs, "parse_attribs: calling handle_href ...");
	handle_href(document, index, neve_node, tag);
      }
    
    else if (strncasecmp(&document[*index], "src", 3) == 0)
      {
	debug (debug_parse_attribs, "parse_attribs: calling handle_src ...");
	handle_src(document, index, neve_node, tag);
      }
    
    else if (strncasecmp(&document[*index], "align", 5) == 0)
      {
	debug (debug_parse_attribs, "parse_attribs: calling handle_align ...");
	handle_align(document, index, neve_node, tag, current_pattribs);
      }
    
    else if (strncasecmp(&document[*index], "size", 4) == 0)
      {
	debug (debug_parse_attribs, "parse_attribs: calling handle_size ...");
	handle_size(document, index, neve_node, tag, current_basefont, 
		    &font_change_implemented, current_pattribs);
      }
    
    else if (strncasecmp(&document[*index], "width", 5) == 0)
      {
	debug (debug_parse_attribs, "parse_attribs: calling handle_width ...");
	handle_width(document, index, neve_node, tag);
      }
	    
    else if (strncasecmp(&document[*index], "height", 6) == 0)
      {
	debug (debug_parse_attribs, "parse_attribs: calling handle_height ...");
	handle_height(document, index, neve_node, tag);
      }    

    else if (strncasecmp(&document[*index], "type", 4) == 0)
      {
	debug (debug_parse_attribs, "parse_attribs: calling handle_type ...");
	handle_type (document, index, neve_node, tag);
      }

    else if (strncasecmp(&document[*index], "color", 5) == 0)
      {
	debug (debug_parse_attribs, "parse_attribs: calling handle_colour ...");
	handle_colour(document, index, neve_node, tag, &font_change_implemented, current_pattribs);
      }


    else
      {
	debug (debug_parse_attribs, 
	       "parse-attribs: Don't recognize this attrib, try and skip from %ld:%s...",
	       *index, get_bit (document, index) ? foo : "NULL");
	handle_unrecognized(document, index);
	debug (debug_parse_attribs, 
	       "parse_attribs: Finished the unrecognized one, now pointing at %ld:%s...",
	       *index, get_bit (document, index) ? foo : "NULL");
      }
  }
  
  if (document[*index] != '\0')
    (*index)++; /* Skip past the last > in the tag */

  debug (debug_parse_attribs,"parse_attribs: Leaving pointing at %ld:%s...",
	 *index, get_bit (document, index) ? foo : "NULL");
  
  return;
}  



void skip_attribs (char *document, long *index)
{
  /* Skips past attriutes which we don't recognize, it'll try to cope sensibly with
   * various different forms and terminate pointing at the right place.
   */
  
  debug (debug_parse_attribs, "skip_attribs: Entered pointing at %ld:%s",
	 *index, get_bit (document, index) ? foo : "NULL");
  do 
    {
      /* Skip until we find a close tag, EOF or a quote (Which we'll
       * presume is the first of a pair
       */
      while (document[*index] != '\0' && document[*index] != '>' && document[*index]!= '"')
	(*index)++;
      
      if (document[*index] != '\0' && document[*index] == '"')
	{
	  (*index)++; /* Skip opening " */
	  
	  /* Presume we're in quoted section so ignore everything
	   * until we find a close quote
	   */
	  while (document[*index] != '\0' && document[*index] != '"')
	    (*index)++;
	  
	  if (document[*index] != '\0')
	    (*index)++; /* Skip closing */
	}
    
    } while (document[*index] != '\0' && document[*index] != '>');
  
  if (document[*index] != '\0')
    (*index)++;

  debug ( debug_parse_attribs, "skip_attribs: Leaving pointing at %ld:%s",
	  *index, get_bit (document, index) ? foo : "NULL");
}

  

void fill_in_alt(neve_in_chains *node)
{
  if (!node->src)
    {
      /* We don't have any src info either, let's let them know their
       * HTML is more broken than a very broken thing.
       */
      const char *error_msg = "{IMAGE:Broken tag, abuse this person ...}";
      node->buf = xmalloc (strlen(error_msg)+1);
      strcpy(node->buf, error_msg);
    }  
  else
    {
      /* If we have a specified URL for the image but no alt text we'll strip the
       * actual filename from the end of the URL and use that since
       * everyone needs some alt text.
       */
      int hpointer; /* Record the position we're skipping back in in the SRC */
      int acounter = 0; /*Record the number of chars we'll need in the node->buf */
      
      hpointer = strlen(node->src)-1;
      acounter = 1; /* We'll always need a '/0' in addition to the text */
      
      debug (debug_parse_attribs, "fill_in_alt: Node's SRC is %s.", node->src);

      while (hpointer  && node->src[hpointer] != '/')
	{
	  debug (debug_parse_attribs, "fill_in_alt: hpointer = %d(%c)", 
		 hpointer, node->src[hpointer]);
	  hpointer--;
	  debug (debug_parse_attribs, "fill_in_alt: hpointer = %d", hpointer);
	}
      
      if (hpointer != 0 || (node->src[hpointer] == '/' && hpointer == 0))
	{
	  hpointer++;
	}
      
      debug ( debug_parse_attribs, "fill_in_alt: hpointer = %d -> %s", 
	      hpointer, node->src+hpointer );
      
      node->buf = xmalloc (strlen(node->src+hpointer)+9);
      sprintf (node->buf, "{IMAGE:%s}", node->src+hpointer);
      
      debug ( debug_parse_attribs, "node->buf = %s", node->buf ) ;
    }
}
