/* YAWN, Yet Another Web Navigator */
/* Copyright (C) 1998, James Troup <jjtroup@comp.brad.ac.uk>
 *                     Lee Willis <lwillis@comp.brad.ac.uk> */

/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* WARNING: This code contains comments and code that may offend, if you are a worry
 * wart or one of those sad little people who complain to Ann Robinson about the use
 * of the word backside on TV then please feel free not to read on .... */

#include <stdio.h>
#include <stdlib.h>
#include "utils.h"
#include "neve.h"
#include "scanner.h"
#include "tagdefs.h"
#include "parser.h"
#include "parse_attribs.h"
#include "neve_operations.h"
#include "staxx.h"

#define foobar (objfound=scan(html_doc, file_index, text_buffer)) 



void add_lbreaks (neve_in_chains *node)
{
  debug (debug_parser, "add_lbreaks: pointing at ...");
  out_neve_type (node);
  
  while (node && (node->type == undefined || node->type == end_link || node->type == start_list_element || node->type == title))
    {
      node = node->previous;
      debug (debug_parser, "add_lbreaks: pointing at ...");
      if (node)
	out_neve_type (node);
      else
	debug (debug_parser, "NULL!");
    }
      
  if (node)
    if (node->type != paragraph_break && node->type != line_break)
      {
	debug (debug_parser, "add_lbreaks: Adding LB" );
	new_neve_node(line_break);
      }
  
}




void add_pbreaks (neve_in_chains *node)
{
  debug (debug_parser, "add_pbreaks: pointing at ...");
  out_neve_type (node);

  while (node && (node->type == undefined || node->type == end_link || node->type == start_list_element || node->type == title))
    {
      node = node->previous;
      debug (debug_parser, "add_pbreaks: pointing at ...");
      if (node)
	out_neve_type (node);
      else
	debug (debug_parser, "NULL!");
    }

  if (node)
    if (node->type == line_break)
      {
	debug (debug_parser, "add_pbreaks: Converting LB to PB." );
	node->type = paragraph_break;
      }
    else if (node->type != paragraph_break)
      {
	debug (debug_parser, "add_pbreaks: Adding PB");
	new_neve_node(paragraph_break);
      }
}




int expect_document (char *html_doc, long *file_index, char *text_buffer)
{
  /* This is the main parsing function  that calls the scanner and the
   * routines to parse the attributes. As it goes along it creates
   * a list of neve nodes with the correct (Hopefully) values to be able
   * to pass the renderer a decent representation of the page.
   */
  
  int hlevel = 0; /* Variable used to construct the header level */
  int objfound = UNRECOGNIZED; /* Used to hold the code of the tag we've found */
  int previous_tag = UNRECOGNIZED; /* Used to hold the previous tag matched */
  int current_basefont_size = 3; /* Stores the current basefont size */
  int dl_top_indent_level = 0; /* Offset to add to get the indent level for dd's */
  
  while ((objfound=scan(html_doc, file_index, text_buffer, 0)) != END_OF_DOC) {

    hlevel = 0;
    switch (objfound)
      {
      case TEXT:
	/* Handle plain text */
	debug (debug_parser, "parser: found text - %s", text_buffer);
	if (neve_list->type == undefined)
	  {
	    neve_list->type = text;
	    debug ( debug_parser, "parser: Converted neve node to type text");
	  }
	if (neve_list->type == horizontal_rule || neve_list->type == line_break || 
	    neve_list->type == paragraph_break)
	  new_neve_node(text);
	debug (debug_parser, "I didn't ever wanna feel ... copying text into node of type ...");
	out_neve_type(neve_list);
	neve_list->buf = xmalloc(strlen(text_buffer)+1);
	strcpy ( neve_list->buf, text_buffer);
	text_buffer [0] = '\0';
	break;

	/* Note for BOLD, ITALIC etc.  These should worry about proper
	 * nesting of tags however bad nesting is such a common
	 * feature that we have simply gone for the idea that <B> sets
	 * the state to bold, until a </B> is found regardless of any
	 * intervening tags. ie
	 * 
	 * <B>Foo<I>bar</B>baz</I>bat
	 *
	 * will be rendered with the foo in bold, the bar in bold
	 * italics, the baz in italics and the bat in plain text as
	 * opposed to the standard which would enforce proper nesting
	 * ie <B><I></I></B> */

	/* We could probably have a generic open and close for bold,
	 * italic, underline, strike, underline, subscript,
	 * superscript etc instead of this hideous code duplication
	 * but it doesn't really matter for now.  */

      case TR:
	add_lbreaks(neve_list);
	skip_attribs(html_doc, file_index);
	break;	

      case BOLD:

	debug (debug_parser, "parser: Found <B>");
	current_pattributes.strong = TRUE; /*Set the current state to reflect bold */
	new_neve_node(undefined); /* Ensure that all new information reflects this */
	neve_list->physical_attributes.strong = TRUE; /* In case we already had a node
							 that was undefined */
	skip_attribs(html_doc, file_index);
	break;
	
      case BOLD|CLOSETAG:

	debug (debug_parser, "parser: Found </B>");
	current_pattributes.strong = FALSE; /* Reset the bold state */
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	else
	  neve_list->physical_attributes.strong = FALSE;
	skip_attribs(html_doc, file_index);
	break;


      case CENTER:
	
	debug (debug_parser, "parser: Found <CENTER>");
	push_align (neve_list); /* Push current alignment onto stack */
	current_pattributes.alignment = centre;
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	else
	  neve_list->physical_attributes.alignment = centre;
	skip_attribs(html_doc, file_index);
	break;
	

      case CENTER|CLOSETAG:

	debug(debug_parser, "parser: Found </CENTER>");
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	pop_align(neve_list); /* Pop last alignment off stack */
	current_pattributes.alignment = neve_list->physical_attributes.alignment;
	skip_attribs(html_doc, file_index);
	break;


      case ITALIC:
      case EMPHASIS:

	debug (debug_parser, "parser: Found <I>");
	current_pattributes.emphasis = TRUE;
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	else
	  neve_list->physical_attributes.emphasis = TRUE;
	skip_attribs(html_doc, file_index);
	break;

      case ITALIC|CLOSETAG:
      case EMPHASIS|CLOSETAG:

	debug (debug_parser, "parser: Found </I>");
	current_pattributes.emphasis = FALSE;
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	else
	  neve_list->physical_attributes.emphasis = FALSE;
	skip_attribs(html_doc, file_index);
	break;

      case UNDERLINE:

	debug (debug_parser, "parser: Found <UL>");
	current_pattributes.underline = TRUE;
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	else
	  neve_list->physical_attributes.underline = TRUE;
	skip_attribs(html_doc, file_index);
	break;

      case UNDERLINE|CLOSETAG:

	debug (debug_parser, "parser: Found </UL>");
	current_pattributes.underline = FALSE;
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	else
	  neve_list->physical_attributes.underline = FALSE;
	skip_attribs(html_doc, file_index);
	break;

      case SUBSCRIPT:

	debug (debug_parser, "parser: Found <SUB>");
	current_pattributes.subscript = TRUE;
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	else
	  neve_list->physical_attributes.subscript = TRUE;
	skip_attribs (html_doc, file_index);
	break;

      case SUBSCRIPT|CLOSETAG:

	debug (debug_parser, "parser: Found </SUB>");
	current_pattributes.subscript = FALSE;
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	else
	  neve_list->physical_attributes.subscript = FALSE;
	skip_attribs(html_doc, file_index);
	break;

      case SUPSCRIPT:
 
	debug (debug_parser, "parser: Found <SUP>");
	current_pattributes.subscript = TRUE;
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	else
	  neve_list->physical_attributes.subscript = TRUE;
	skip_attribs (html_doc, file_index);
	break;

      case SUPSCRIPT|CLOSETAG:

	debug (debug_parser, "parser: Found </SUP>");
	current_pattributes.superscript = FALSE;
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	else
	  neve_list->physical_attributes.superscript = FALSE;
	skip_attribs(html_doc, file_index);
	break;

	

      case DL:

	debug (debug_parser, "parser: Found <DL>");
	push_list (neve_list);
	dl_top_indent_level = current_lattributes.list_indent_level;
	current_lattributes.item_number = 0;
	current_lattributes.dl_list_element = TRUE;
	current_lattributes.ul_list_element = FALSE;
	current_lattributes.ol_list_element = FALSE;
	
	{
	  neve_in_chains *node = neve_list;
	  
	  while (node->previous && (node->type == undefined || node->type == end_link || node->type == start_list_element))
	    node = node->previous;
	 
	  debug (debug_parser, "parser.c: Trying to add PB for start of list, pointing at ...");
	  out_neve_type (node);
	  if (!node->previous)
	    new_neve_node(paragraph_break);
	  else if (node->type == line_break)
	    node->type = paragraph_break;
	  else
	    new_neve_node(paragraph_break);
	}

	debug (debug_parser, "parser: Indent =%d, item_number=%d",
	       current_lattributes.list_indent_level, current_lattributes.item_number);
	new_neve_node(undefined);
	neve_list->logical_attributes.list_indent_level = current_lattributes.list_indent_level;
	neve_list->logical_attributes.item_number = 0;	
	neve_list->logical_attributes.dl_list_element = TRUE;
	neve_list->logical_attributes.ol_list_element = FALSE;
	neve_list->logical_attributes.ul_list_element = FALSE;
	parse_attribs (html_doc, file_index, neve_list, ULIST, 
		       &current_basefont_size, &current_pattributes);
	break;


      case DT:

	debug (debug_parser, "parser: Found <DT>");

	{
	  neve_in_chains *node = neve_list;

	  debug (debug_parser, "parser: trying to add line_break for start_list_element");
	  while (node->previous && 
		 (node->type == undefined || node->type == end_link || node->type == start_list_element) && 
		 node->logical_attributes.item_number != 0)
	    node = node->previous;

	  debug (debug_parser, "parser: pointing at ...");
	  out_neve_type (node);
	  
	  if (node->logical_attributes.item_number != 0)
	    {
	      if (node->type == paragraph_break)
		node->type = line_break;
	      else if (node->type != line_break)
		new_neve_node(line_break);
	    }
	}
	
	current_lattributes.item_number++;
	current_lattributes.list_indent_level = dl_top_indent_level;
	new_neve_node(start_list_element);
	new_neve_node(undefined);
	parse_attribs (html_doc, file_index, neve_list, LIST, 
		       &current_basefont_size, &current_pattributes);
	break;

	
      case DD:

	debug (debug_parser, "parser: Found <DD>");

	{
	  neve_in_chains *node = neve_list;

	  debug (debug_parser, "parser: trying to add line_break for start_list_element");
	  while (node->previous && 
		 (node->type == undefined || node->type == end_link || node->type == start_list_element) && 
		 node->logical_attributes.item_number != 0)
	    node = node->previous;

	  debug (debug_parser, "parser: pointing at ...");
	  out_neve_type (node);
	  
	  if (node->logical_attributes.item_number != 0)
	    {
	      if (node->type == paragraph_break)
		node->type = line_break;
	      else if (node->type != line_break)
		new_neve_node(line_break);
	    }
	}
	
	current_lattributes.item_number++;
	current_lattributes.list_indent_level = dl_top_indent_level + 1;
	new_neve_node(start_list_element);
	new_neve_node(undefined);
	parse_attribs (html_doc, file_index, neve_list, LIST, 
		       &current_basefont_size, &current_pattributes);
	break;


      case DL|CLOSETAG:

 	debug (debug_parser, "parser: Found </DL>");
	pop_list(neve_list);
	current_lattributes.list_indent_level = dl_top_indent_level - 1;
	if (current_lattributes.list_indent_level)
	  current_lattributes.item_number = ++(neve_list->logical_attributes.item_number);
	else
	  current_lattributes.item_number = (neve_list->logical_attributes.item_number);
	current_lattributes.ol_list_element = neve_list->logical_attributes.ol_list_element;
	current_lattributes.ul_list_element = neve_list->logical_attributes.ul_list_element;
	current_lattributes.dl_list_element = neve_list->logical_attributes.dl_list_element;
	skip_attribs(html_doc, file_index);
	if (!current_lattributes.list_indent_level)
	  add_lbreaks(neve_list);
 	new_neve_node(undefined);
	break;

	
      case ULIST:

	debug (debug_parser, "parser: Found <UL>");
	push_list(neve_list);
	current_lattributes.list_indent_level++;
	current_lattributes.item_number = 0;
	current_lattributes.ul_list_element = TRUE;
	current_lattributes.ol_list_element = FALSE;
	current_lattributes.dl_list_element = FALSE;
	
	{
	  neve_in_chains *node = neve_list;
	  
	  while (node->previous && (node->type == undefined || node->type == end_link || node->type == start_list_element))
	    node = node->previous;
	 
	  debug (debug_parser, "parser.c: Trying to add PB for start of list, pointing at ...");
	  out_neve_type (node);
	  if (!node->previous)
	    new_neve_node(paragraph_break);
	  else if (node->type == line_break)
	    node->type = paragraph_break;
	  else
	    new_neve_node(paragraph_break);
	}

 	debug (debug_parser, "parser: Indent=%d, item_number=%d",
	       current_lattributes.list_indent_level, current_lattributes.item_number);
	new_neve_node(undefined); 
	neve_list->logical_attributes.list_indent_level = current_lattributes.list_indent_level;
	neve_list->logical_attributes.item_number = 0;	
	neve_list->logical_attributes.ul_list_element = TRUE;
	neve_list->logical_attributes.ol_list_element = FALSE;
	neve_list->logical_attributes.dl_list_element = FALSE;
	parse_attribs (html_doc, file_index, neve_list, ULIST, 
		       &current_basefont_size, &current_pattributes);
	current_lattributes.ul_bullet_type = neve_list->logical_attributes.ul_bullet_type;
	break;
	
      case OLIST:

	debug (debug_parser, "parser: Found <OL>");
	push_list(neve_list);
	current_lattributes.list_indent_level++;
	current_lattributes.item_number = 0;
	current_lattributes.ol_list_element = TRUE;
	current_lattributes.ul_list_element = FALSE;
	current_lattributes.dl_list_element = FALSE;
	
	{
	  neve_in_chains *node = neve_list;
	  
	  while (node->previous && (node->type == undefined || node->type == end_link || node->type == start_list_element))
	    node = node->previous;
	  
	  debug (debug_parser, "parser.c: Trying to add PB for start of list, pointing at ...");
	  out_neve_type (node);
	  if (node->type == line_break)
	    {
	      debug (debug_parser, "parser.c: Altering LB to PB.");
	      node->type = paragraph_break;
	    }
	  else
	    new_neve_node(paragraph_break);
	}

 	debug (debug_parser, "parser: Indent=%d, item_number=%d",
	       current_lattributes.list_indent_level, current_lattributes.item_number);
 	new_neve_node(undefined); 
	neve_list->logical_attributes.list_indent_level = current_lattributes.list_indent_level;
	neve_list->logical_attributes.item_number = 0;
	neve_list->logical_attributes.ol_list_element = TRUE;
	neve_list->logical_attributes.ul_list_element = FALSE;
	neve_list->logical_attributes.dl_list_element = FALSE;
	parse_attribs (html_doc, file_index, neve_list, OLIST, 
		       &current_basefont_size, &current_pattributes);
	current_lattributes.ol_number_type = neve_list->logical_attributes.ol_number_type;
	break;

      case ULIST|CLOSETAG:
      case OLIST|CLOSETAG:

 	debug (debug_parser, "parser: Found %s", (objfound == (ULIST|CLOSETAG)) ? "/UL" : "/OL");
	pop_list(neve_list);
	current_lattributes.list_indent_level--;
	if (current_lattributes.list_indent_level)
	  current_lattributes.item_number = ++(neve_list->logical_attributes.item_number);
	else
	  current_lattributes.item_number = (neve_list->logical_attributes.item_number);
	current_lattributes.ol_list_element = neve_list->logical_attributes.ol_list_element;
	current_lattributes.ul_list_element = neve_list->logical_attributes.ul_list_element;
	current_lattributes.dl_list_element = neve_list->logical_attributes.dl_list_element;
	skip_attribs(html_doc, file_index);
	if (!current_lattributes.list_indent_level)
	  add_lbreaks(neve_list);
 	new_neve_node(undefined);
	break;

      case LIST:

	debug (debug_parser, "parser: Found <LI>");

	{
	  neve_in_chains *node = neve_list;

	  debug (debug_parser, "parser: trying to add line_break for start_list_element");
	  while (node->previous && 
		 (node->type == undefined || node->type == end_link || node->type == start_list_element) && 
		 node->logical_attributes.item_number != 0)
	    node = node->previous;

	  debug (debug_parser, "parser: pointing at ...");
	  out_neve_type (node);
	  
	  if (node->logical_attributes.item_number != 0)
	    {
	      if (node->type == paragraph_break)
		node->type = line_break;
	      else if (node->type != line_break)
		new_neve_node(line_break);
	    }
	}
	
	current_lattributes.item_number++;
	new_neve_node(start_list_element);
	new_neve_node(undefined);
	parse_attribs (html_doc, file_index, neve_list, LIST, 
		       &current_basefont_size, &current_pattributes);
	break;
	

      case ANCHOR:

	debug (debug_parser, "parser: Found <A>");
	current_lattributes.link = TRUE;
	new_neve_node(undefined);
	neve_list->logical_attributes.link = TRUE;
	parse_attribs (html_doc, file_index, neve_list, ANCHOR, 
		       &current_basefont_size, &current_pattributes);
	if (!neve_list->href)
	  {
	    current_lattributes.link = FALSE;
	    neve_list->logical_attributes.link = FALSE;
	  }
	break;

	
      case ANCHOR|CLOSETAG:

	debug (debug_parser, "parser: Found </A>");
	current_lattributes.link = FALSE;
	new_neve_node(end_link); /* Makes rendering easier */
 	new_neve_node(undefined);
	skip_attribs (html_doc, file_index);
	break;


      case PARAGRAPH: 

	debug (debug_parser, "parser: Found <P>");
	add_pbreaks(neve_list);
	push_align (neve_list); /* Push current alignment onto stack */
	parse_attribs (html_doc, file_index, neve_list, PARAGRAPH, &current_basefont_size, 
		       &current_pattributes);
	break;


      case PARAGRAPH|CLOSETAG:

	debug (debug_parser, "parser: Found </P>");
	add_pbreaks(neve_list);
	pop_align (neve_list); /* Pop alignment off stack */
	current_pattributes.alignment = neve_list->physical_attributes.alignment;
	skip_attribs (html_doc, file_index);
 	new_neve_node(undefined); 
	break;


      case IMAGE:

	debug (debug_parser, "parser: Found <IMG>");
	new_neve_node(image);
        /* Ensure that TEXT/P/DIV align values are not carried into the image node */
	neve_list->physical_attributes.alignment = 0;
   	parse_attribs (html_doc, file_index, neve_list, IMAGE, 
		       &current_basefont_size, &current_pattributes);
	/* We'd like to put something as ALT text if it isn't given by the doc itself
	 * Decided on {IMAGE:filename}
	 */
	if (!neve_list->buf)
	  fill_in_alt(neve_list);
	new_neve_node(undefined);
	break;


      case IMAGE|CLOSETAG:

	debug (debug_parser, "parser: Found </IMG>, ignoring!");
	skip_attribs (html_doc, file_index);  /* Not allowed, but just to be on the safe side */
	break;

	
      case BASEFONT:

	debug (debug_parser, "parser: Found <BASEFONT>");
	parse_attribs (html_doc, file_index, neve_list, BASEFONT, 
		       &current_basefont_size, &current_pattributes);
	current_pattributes.font_size = current_basefont_size;
	break;

	
      case FONT:
	
	debug (debug_parser, "parser: Found <FONT>");
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	parse_attribs (html_doc, file_index, neve_list, FONT, 
		       &current_basefont_size, &current_pattributes);
       	current_pattributes.font_size = neve_list->physical_attributes.font_size;
	current_pattributes.font_colour = neve_list->physical_attributes.font_colour;
	break;

	
      case FONT|CLOSETAG:
	
	debug (debug_parser, "parser: Found </FONT>");
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	pop_font (neve_list);
	current_pattributes.font_size = neve_list->physical_attributes.font_size;
	current_pattributes.font_colour = neve_list->physical_attributes.font_colour;
	skip_attribs (html_doc, file_index);
	break;
	
	/* This is very very bad, but it is also the most efficient way of doing things */
	/* Do NOT, and i repeat NOT reorder or insert any additional cases until after the H1 */
      case H6:
	hlevel++;
      case H5:
	hlevel++;
      case H4:
	hlevel++;
      case H3:
	hlevel++;
      case H2:
	hlevel++;
      case H1:
	hlevel++;

	debug (debug_parser, "parser: Found a <H%d>", hlevel);
	add_pbreaks(neve_list);
	push_header(neve_list, current_lattributes.header_level);
	current_lattributes.header_level = hlevel;
	neve_list->logical_attributes.header_level = hlevel;
	new_neve_node(undefined);
	parse_attribs (html_doc, file_index, neve_list, H1, 
		       &current_basefont_size, &current_pattributes);
	break;
	/* End evil code */

	
      case H1|CLOSETAG:
      case H2|CLOSETAG:
      case H3|CLOSETAG:
      case H4|CLOSETAG:
      case H5|CLOSETAG:
      case H6|CLOSETAG:
	
	debug (debug_parser, "parser: Found a close header tag");
	add_lbreaks(neve_list);
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	pop_header(neve_list);
	current_lattributes.header_level = neve_list->logical_attributes.header_level;
	current_pattributes.alignment = neve_list->physical_attributes.alignment;
	skip_attribs (html_doc, file_index);
	break;

	
      case HRULE:

	debug (debug_parser, "parser: Found a <HR>");
	add_lbreaks (neve_list);
	parse_attribs (html_doc, file_index, neve_list, HRULE, 
		       &current_basefont_size, &current_pattributes);
	new_neve_node (horizontal_rule);
	break; /* YARK should be followed by carriage return */
	

      case BR:
	
	debug (debug_parser, "parser: Found a <BR>");
	new_neve_node(line_break);
	parse_attribs (html_doc, file_index, neve_list, BR, 
		       &current_basefont_size, &current_pattributes);
	/* new_neve_node(undefined); */
	break;
	
      case TITLE:

	debug (debug_parser, "parser: Found a <TITLE>");
	new_neve_node(title);
	skip_attribs (html_doc, file_index);
	break;

      case TITLE|CLOSETAG:

	debug (debug_parser, "parser: Found a </TITLE>");
 	new_neve_node(undefined); 
	skip_attribs (html_doc, file_index);
	break;

      case BLOCKQUOTE:

	debug (debug_parser, "parser: Found a <BLOCKQUOTE>");
	current_pattributes.blockquote = TRUE;
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	else
	  neve_list->physical_attributes.blockquote = TRUE;
	skip_attribs (html_doc, file_index);
	break;

      case BLOCKQUOTE|CLOSETAG:
	
	debug (debug_parser, "parser: Found </BLOCKQUOTE>");
	current_pattributes.blockquote = FALSE;
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	else
	  neve_list->physical_attributes.blockquote = FALSE;
	skip_attribs(html_doc, file_index);
	break;

      case ADDRESS:

	debug (debug_parser, "parser: Found <ADDRESS>");
	current_pattributes.emphasis = TRUE;
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	else
	  neve_list->physical_attributes.emphasis = TRUE;
	skip_attribs(html_doc, file_index);
	break;

      case ADDRESS|CLOSETAG:

	debug (debug_parser, "parser: Found </ADDRESS>");
	current_pattributes.emphasis = FALSE;
	if (neve_list->type != undefined)
	  new_neve_node(undefined);
	else
	  neve_list->physical_attributes.emphasis = FALSE;
	skip_attribs(html_doc, file_index);
	break;
	
      case COMMENT:
	break;

      default:
	debug (debug_parser, "parser: something we don't deal with DOH!");
	if (neve_list->type == text)
	  new_neve_node(undefined);
	skip_attribs (html_doc, file_index);
	break;
      }
    if (objfound <= CLOSETAG)
      while (is_whitespace(html_doc[*file_index]))
	{
	  debug (debug_parser, "parser: Skipping \\n after start tag ..." );
	  (*file_index)++;
	}
    previous_tag = objfound;

  }    
  return 0;
}



int parse_html (char *html_doc, GtkLisa *lisa)
{
  /* This is the entry point that initialises all our variables for a
   * new document */

  char *text_buffer;
  long file_index = 0;

  text_buffer = xmalloc(BUF_SIZE);
    
  /* Set up the Logical attributes to start with */
  current_lattributes.header_level = 0;
  current_lattributes.link = FALSE;
  current_lattributes.link_visited = FALSE;
  current_lattributes.ul_list_element = FALSE;
  current_lattributes.ol_list_element = FALSE;
  current_lattributes.dl_list_element = FALSE;
  current_lattributes.list_indent_level = 0;
  /* Set up the Physical attributes */
  current_pattributes.font_size = 3;
  current_pattributes.strong = FALSE;
  current_pattributes.emphasis = FALSE;
  current_pattributes.code = FALSE;
  current_pattributes.alignment = left;
  current_pattributes.underline = FALSE;
  current_pattributes.subscript = FALSE;
  current_pattributes.superscript = FALSE;
  current_pattributes.strike = FALSE;
  current_pattributes.font_colour.red = 0;
  current_pattributes.font_colour.green = 0;
  current_pattributes.font_colour.blue = 0;
  current_pattributes.blockquote = FALSE;
  
  /* Set up the dummy Neve element */
  
  new_neve_node(undefined);
  neve_head = neve_list; 
  expect_document(html_doc, &file_index, text_buffer);
  output_list (neve_head);
  debug ( debug_parser, 
	  "parser: Succesfully finished parsing, passing struct to rendered" ) ;
  lisa_take_neve (neve_head, lisa);
  
  return 0;
}
