/* YAWN, Yet Another Web Navigator */
/* Copyright (C) 1998, James Troup <jjtroup@comp.brad.ac.uk>
 *                     Lee Willis <lwillis@comp.brad.ac.uk> */

/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include "utils.h"
#include "staxx.h"

/* This module deals with push and pop operations for the variouus
 * stacks needed to keep track of the state of font size/colour,
 * header levels, paragraph alignments, and list indention and
 * numbering levels. This allows nested <FONT> and <HEADER> tags even
 * thought the latter is incorrect HTML. It also allows correct
 * indentation of lists and correct numbering when dealing with nested
 * lists.
 */

void push_align (neve_in_chains *neve)
{
  if (!align_stack)
    {
      debug (debug_parser, "staxx: Creating alignment stack.");
      align_stack = xmalloc (sizeof(*align_stack));
      align_stack->previous = NULL;
      /* YARK should we set NEXT to null as well or is it unnecessary?
       * Seems to work as is but that don't mean shit frankly.
       */
    }
  else
    {
      debug (debug_parser, "staxx: Extending alignment stack.");
      align_stack->next = xmalloc (sizeof(*align_stack));
      align_stack->next->previous = align_stack;
      align_stack = align_stack->next;
    }
  align_stack->alignment = neve->physical_attributes.alignment;
  debug (debug_parser, "staxx: Pushed alignment %d onto align stack.", align_stack->alignment);
  align_stack->next = NULL;  /* YARK, do we need this? */
}


void pop_align (neve_in_chains *neve)
{
  align_node *temp;

  if (!align_stack)
    {
      debug (debug_parser, "staxx: No align_stack available, returning.");
      neve->physical_attributes.alignment = left; /* DEFAULT */
      return;
    }
  else
    {
      neve->physical_attributes.alignment = align_stack->alignment;
      debug (debug_parser, "staxx: Popped alignment %d off p_stack.", align_stack->alignment);
      temp = align_stack;
      align_stack = align_stack->previous;
      free (temp);
      if (align_stack)
	align_stack->next = NULL;
    }
}

/* void push_p (neve_in_chains *neve) */
/* { */
/*   if (!p_stack) */
/*     { */
/*       debug (debug_parser, "staxx: Creating p_stack."); */
/*       p_stack = xmalloc (sizeof(*p_stack)); */
/*       p_stack->previous = NULL; */
/*     } */
/*   else */
/*     { */
/*       debug (debug_parser, "staxx: Extending p_stack."); */
/*       p_stack->next = xmalloc(sizeof(*p_stack)); */
/*       p_stack->next->previous = p_stack; */
/*       p_stack = p_stack->next; */
/*     } */
/*   p_stack->alignment = neve->physical_attributes.alignment; */
/*   debug (debug_parser, "staxx: Pushed alignment %d onto p_stack.", p_stack->alignment); */
/*   p_stack->next = NULL; */
/* } */

/* void pop_p (neve_in_chains *neve) */
/* { */
/*   p_node *temp; */

/*   if (!p_stack) */
/*     { */
/*       debug (debug_parser, "staxx: No p_stack available, returning."); */
/*       return; */
/*     } */
/*   else */
/*     { */
/*       neve->physical_attributes.alignment = p_stack->alignment; */
/*       debug (debug_parser, "staxx: Popped alignment %d off p_stack.", p_stack->alignment); */
/*       temp = p_stack; */
/*       p_stack = p_stack->previous; */
/*       free (temp); */
/*       if (p_stack) */
/* 	p_stack->next = NULL; */
/*     } */
/* } */


void push_font (neve_in_chains *neve, boolean colour, boolean size)
{
  if (!font_stack)
    {
      debug (debug_parser, "staxx: Creating font_stack.");
      font_stack = xmalloc (sizeof(*font_stack));
      font_stack->previous = NULL;
    }
  else
    {
      debug (debug_parser, "staxx: Extending font stack.");
      font_stack->next = xmalloc(sizeof(*font_stack));
      font_stack->next->previous = font_stack;
      font_stack = font_stack->next;
    }
  font_stack->node_with_old_values = neve;
  if (colour)
    {
      debug (debug_parser, "staxx: Colour change.");
      font_stack->colour_change = colour;
    }
  if (size)
    {
      debug (debug_parser, "staxx: Size change.");
      font_stack->size_change = size;
    }
  font_stack->next = NULL;
}				 


void pop_font (neve_in_chains *neve)
{
  font_node *temp;
  
  if (!font_stack)
    {
      debug (debug_parser, "staxx: No font_stack available, returning.");
      return;
    }
  else 
    {
      if (font_stack->colour_change)
	{
	  neve->physical_attributes.font_colour = 
	    font_stack->node_with_old_values->physical_attributes.font_colour;
	  debug (debug_parser, "staxx: Popped colour %d.%d.%d off stack.", 
		 neve->physical_attributes.font_colour.red,
		 neve->physical_attributes.font_colour.green,
		 neve->physical_attributes.font_colour.blue);
	} 
	 
      if (font_stack->size_change)
	{
	  neve->physical_attributes.font_size = 
	    font_stack->node_with_old_values->physical_attributes.font_size;
	  debug (debug_parser, "staxx: Popped size %d off stack.",
		 neve->physical_attributes.font_size);
	}
      
      temp = font_stack;
      font_stack = font_stack->previous;
      free (temp);
      if (font_stack)
	font_stack->next = NULL;
    }
}


void push_header (neve_in_chains *neve, int level)
{
  if (!header_stack)
    {
      debug (debug_parser, "staxx: Creating header_stack.");
      header_stack = xmalloc (sizeof(*header_stack));
      header_stack->previous = NULL;
    }
  else
    {
      debug (debug_parser, "staxx: Extending header_stack.");
      header_stack->next = xmalloc(sizeof(*header_stack));
      header_stack->next->previous = header_stack;
      header_stack = header_stack->next;
    }
  header_stack->alignment = neve->physical_attributes.alignment;
  header_stack->level = level;
  debug(debug_parser, "staxx: Pushed align=%d, level=%d onto header_stack.",
	header_stack->alignment,
	header_stack->level);
  header_stack->next = NULL;
}



void pop_header (neve_in_chains *neve)
{
  header_node *temp;

  if (!header_stack)
    {
      debug (debug_parser, "staxx: No header_stack available, returning.");
      return;
    }
  else
    {
      neve->logical_attributes.header_level = header_stack->level;
      neve->physical_attributes.alignment = header_stack->alignment;
      debug (debug_parser, "staxx: Popped align=%d, level=%d off header_stack.",
	     neve->physical_attributes.alignment,
	     neve->logical_attributes.header_level);
      temp = header_stack;
      header_stack = header_stack->previous;
      free (temp);
      
      if (header_stack)
	header_stack->next = NULL;
    }
}
	


void push_list (neve_in_chains *neve)
{
  if (!list_stack)
    {
      debug (debug_parser, "staxx: Creating list_stack.");
      list_stack = xmalloc (sizeof(*list_stack));
      list_stack->previous = NULL;
    }
  else
    {
      debug (debug_parser, "staxx: Extending list_stack.");
      list_stack->next = xmalloc(sizeof(*list_stack));
      list_stack->next->previous = list_stack;
      list_stack = list_stack->next;
    }
  list_stack->ol = neve->logical_attributes.ol_list_element;
  list_stack->ul = neve->logical_attributes.ul_list_element;
  list_stack->dl = neve->logical_attributes.dl_list_element;
  list_stack->indent_level = neve->logical_attributes.list_indent_level;
  list_stack->count = neve->logical_attributes.item_number;
  debug (debug_parser, "staxx: Pushed element onto stack ...");
  if (list_stack->ol)
    debug (debug_parser, "staxx: ordered list");
  if (list_stack->ul)
    debug (debug_parser, "staxx: unordered list");
  if (list_stack->dl)
    debug (debug_parser, "staxx: definition list");
  debug (debug_parser, "staxx: level=%d, count=%d",
	 list_stack->indent_level,
	 list_stack->count);
  list_stack->next = NULL;
}



void pop_list (neve_in_chains *neve)
{
 list_node *temp;

  if (!list_stack)
    {
      debug(debug_parser, "staxx: No list_stack available, returning.");
      return;
    }
  else
    {
      neve->logical_attributes.list_indent_level = list_stack->indent_level;
      neve->logical_attributes.item_number = list_stack->count;
      neve->logical_attributes.ol_list_element = list_stack->ol;
      neve->logical_attributes.ul_list_element = list_stack->ul;
      neve->logical_attributes.dl_list_element = list_stack->dl;
      debug (debug_parser, "staxx: Popped element from stack ...");
      if (list_stack->ol)
	debug (debug_parser, "staxx: ordered list");
      if (list_stack->ul)
	debug (debug_parser, "staxx: unordered list");
      if (list_stack->dl)
	debug (debug_parser, "staxx: definition list");
      debug (debug_parser, "staxx: level=%d, count=%d",
	     list_stack->indent_level,
	     list_stack->count);
      temp = list_stack;
      list_stack = list_stack->previous;
      free (temp);
      
      if (list_stack)
	list_stack->next = NULL;
    }
}



      
void hack_fstack_size (void)
{
  if (font_stack)
    font_stack->size_change = TRUE;
}


void hack_fstack_colour (void)
{
  if (font_stack)
    font_stack->colour_change = TRUE;
}
